Ext.define('DSS.component.Editor', {
    extend : 'Ext.window.Window',

    /** Textfield holding the event name */
    editControls : {},
    buttonSave : null,
    buttonCancel : null,
    oTabPanelParent : null,
    innerHeight : 427,
    oTabPanels:{},
    makeControl : function(subConfig,finTab) {
        var oControl;
        switch (subConfig.type) {
            case 'combobox' : {
            	
            	oControl = Ext.create('DSS.component.ComboboxControl', {
            		data:subConfig.data,
            		labelWidth:subConfig.labelWidth,
            		storevalueid:subConfig.storevalueid,
            		storedisplayid:subConfig.storedisplayid,
            		label:subConfig.label,
            		storeid:subConfig.storeid,
                    myID:subConfig.id
                });

                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 27;
                this.editControls[subConfig.id] = oControl;
                oControl.dssEditor=this;
                return oControl;
                
                break;
            }
            case 'radio' : {
            	
            	oControl = Ext.create('DSS.component.RadioControl', {
            		data:subConfig.data,            		
            		labelWidth:subConfig.labelWidth,
            		label:subConfig.label,
            		radioid:subConfig.id,
                    myID:subConfig.id
                });

            	
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 27 * subConfig.data.length;
                this.editControls[subConfig.id] = oControl;
                oControl.dssEditor=this;
                return oControl;
                
                break;
            }
            case 'hidden': {
                var oPseudoControl = {
                    value : null
                };
                oPseudoControl.setValue = function(oValue) {
                    this.value = oValue;
                };
                oPseudoControl.reset = function(oValue) {
                    this.value = null;
                };
                oPseudoControl.getValue = function() {
                    return this.value;
                };
                oPseudoControl.isValid = function() {
                    return true;
                };
                this.editControls[subConfig.id] = oPseudoControl;
                return null;
                break;
            }
            case 'custom': {
                oControl = subConfig.control;
                if ((subConfig.flex == null) || (!subConfig.flex)) {
                	try  {
                		var iHeight=oControl.getHeight();
                		this.innerHeight -= iHeight;
                	} 
                	catch (e) {
                        this.innerHeight -= 22;
                	}
                }
                this.editControls[subConfig.id] = oControl;
                oControl.dssEditor=this;
                return oControl;
                break;
            }

            case 'timespan': {
                oControl = Ext.create('DSS.component.TimeSpanControl', {
                    name : subConfig.id,
            		labelWidth:subConfig.labelWidth,
                    fieldLabel : subConfig.label,
                    hideHour: subConfig.hideHour,
                    hideMinute:subConfig.hideMinute,
                    hideSecond:subConfig.hideSecond,
                    myID:subConfig.id
                });
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 32;
                this.editControls[subConfig.id] = oControl;
                oControl.dssEditor=this;
                if (finTab)
                	oControl.setMargin("3 0 0 2");
                return oControl;
                break;
            }
            case 'deviceSelection': {
                oControl = Ext.create('DSS.component.DeviceSelector', {
                    filterFunc:subConfig.filterFunc,
                    specialDeviceRenderer:subConfig.specialDeviceRenderer,
                    myID:subConfig.id
                });
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 27;
                this.editControls[subConfig.id] = oControl;
                oControl.dssEditor=this;
                return oControl;
                break;
            }
            case 'number': {
                oControl = Ext.create('Ext.form.field.Number', {
                    name : subConfig.id,
                    allowBlank : false,
                    blankText : _('This field is required'),
                    invalidText : _('The value in this field is invalid'),
                    fieldLabel : subConfig.label,
                    width : 200,
                    step : 1,
                    value : 0,
                    minValue : subConfig.minValue,
                    maxValue : subConfig.maxValue,
                    myID:subConfig.id
                });
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 27;
                this.editControls[subConfig.id] = oControl;
                oControl.dssEditor=this;
                return oControl;
                break;
            }
            case 'groupbox': {
                var oSubcontrols = [];
                if (subConfig.content != null)
                    for ( var iIndex = 0; iIndex < subConfig.content.length; iIndex++)
                        oSubcontrols.push(this.makeControl(subConfig.content[iIndex]),finTab);

                oControl = Ext.create('Ext.form.FieldSet', {
                    title : subConfig.label,
                    items : oSubcontrols,
                    padding: "0 5 10 0",
                    margin: 5
                });
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 27;

                oControl.dssEditor=this;
                return oControl;
                break;
            }
            case 'checkbox': {
                oControl = Ext.create('Ext.form.field.Checkbox', {
                    name : subConfig.id,
                    fieldLabel : subConfig.title,
                    boxLabel : subConfig.label,
                    allowBlank : false,
                    labelSeparator: ( subConfig.title=='&nbsp;')?'':':',
                            myID:subConfig.id
                });
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 22;
                this.editControls[subConfig.id] = oControl;
                oControl.dssEditor=this;
                return oControl;
                break;
            }
            case 'text': {
            	if (subConfig.labelWidth==null)
            		subConfig.labelWidth=100;
                oControl = Ext.create('DSS.component.Textedit', {
                    name : subConfig.id,
                    fieldLabel : subConfig.label,
            		labelWidth:subConfig.labelWidth,
                    allowBlank : false,
                    blankText : _('This field is required'),
                    invalidText : _('The value in this field is invalid'),
                    width : 600,
                    inputWidth : 500,
                    myID:subConfig.id
                });
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 27;
                this.editControls[subConfig.id] = oControl;
                oControl.dssEditor=this;
                return oControl;
                break;
            }
            case 'date': {
                oControl = Ext.create('DSS.component.Dateedit', {
                    name : subConfig.id,
                    fieldLabel : subConfig.label,
            		labelWidth:subConfig.labelWidth,
                    allowBlank : false,
                    width : 600,
                    myID:subConfig.id
                });
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 27;
                this.editControls[subConfig.id] = oControl;
                oControl.dssEditor=this;
                return oControl;
                break;
            }
            case 'weekdaySelection': {
                oControl = Ext.create('DSS.component.WeekdaySelection', {
                    name : subConfig.id,
            		labelWidth:subConfig.labelWidth,
                    label : subConfig.label,
                    width : 800,
                    myID:subConfig.id
                });
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 27;
                this.editControls[subConfig.id] = oControl;
                oControl.dssEditor=this;
                return oControl;
                break;
            }
            case 'ActionConfigurator': {
                oControl = Ext.create('DSS.component.ActionConfigurator', {
                    height : 350,
                    noHighlevel : subConfig.noHighlevel,
                    withDelay: subConfig.withDelay,
                    allowNoAction : subConfig.allowNoAction,
                    enableUsergroup:subConfig.enableUsergroup,
                    userStates:subConfig.userStates,
                    myID:subConfig.id
                });
                this.editControls[subConfig.id] = oControl;
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 350;
                oControl.dssEditor=this;
                return oControl;
                break;
            }
            case 'ConditionConfigurator': {
                oControl = Ext.create('DSS.component.ConditionConfigurator', {
                    height : 350,
                    conditions:subConfig.conditions,
                    myID:subConfig.id
                });
                this.editControls[subConfig.id] = oControl;
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 350;
                oControl.dssEditor=this;
                return oControl;
                break;
            }
            case 'TriggerConfigurator': {
                oControl = Ext.create('DSS.component.TriggerConfigurator', {
                    height : 350,
                    enableUsergroup:subConfig.enableUsergroup,
                    userStates:subConfig.userStates,
                    myID:subConfig.id
                });
                this.editControls[subConfig.id] = oControl;
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 350;
                oControl.dssEditor=this;
                return oControl;
                break;
            }
            case 'MessageControl': {
                oControl = Ext.create('DSS.component.MessageEditor', {
            		labelWidth:subConfig.labelWidth,
                    title : subConfig.title,
                    myID:subConfig.id
                });
                this.editControls[subConfig.id] = oControl;
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 150;
                
                oControl.dssEditor=this;
                return oControl;
                break;
            }
            case 'TitterMsgControl' : 
            {
                oControl = Ext.create('DSS.component.TitterMessageEditor', {
            		labelWidth:subConfig.labelWidth,
                    title : subConfig.title,
                    myID:subConfig.id
                });
                this.editControls[subConfig.id] = oControl;
                if ((subConfig.flex == null) || (!subConfig.flex))
                    this.innerHeight -= 150;
                
                oControl.dssEditor=this;
                return oControl;
                break;
            }
            case 'timeReference' :{
                oControl = Ext.create('DSS.component.TimeReference', {
                    name : subConfig.id,
            		labelWidth:subConfig.labelWidth,
                    label : subConfig.label,
                    width : 800,
                    randomize: subConfig.randomize,
                    additionalAbsoluteValues: subConfig.additionalAbsoluteValues,
                    referenceChangeListener:subConfig.referenceChangeListener,
                    myID:subConfig.id
                });
                if ((subConfig.flex == null) || (!subConfig.flex)) {
                    if (subConfig.randomize)
                        this.innerHeight -= (54 + 26);
                    else
                        this.innerHeight -= 54;                    
                }
                    
                this.editControls[subConfig.id] = oControl;
                oControl.dssEditor=this;
                return oControl;
                break;
            }
        }
        return Ext.create('Ext.panel.Panel', {
            border : '0'
        });
    },
    defaultFirstTabOnNew:0,
    defaultFirstTabOnEdit:0,
    constructor : function(config) {
        config.closeAction = 'hide';
        config.modal = true; 
        config.height = 500;
        config.width = 800;
        config.layout = 'fit';
        config.appName = '';
        config.resizable=false;
        this.defaultFirstTabOnNew=0;
        this.defaultFirstTabOnEdit=0;

        this.editControls={};
        this.innerHeight = 427;
        //this.mixins.observable.constructor.call(config);

        var me = this;
        var sCancelText=_('cancel');
        if (config.cancelTitle!=null) {
            sCancelText=config.cancelTitle;
        }
        var sSaveText=_('save');
        if (config.saveTitle!=null) {
            sSaveText=config.saveTitle;
        }
        
        if ((config.hideCancel==null) ||(config.hideCancel==false)) { 
            me.buttonCancel = Ext.create('Ext.Button', {
                text : sCancelText,
                handler : function() {
                    me.fireEvent('cancel');
                    me.hide();
                }
            });
        }
        if ((config.hideSave==null) ||(config.hideSave==false)) {
            me.buttonSave = Ext.create('Ext.Button', {
                text : sSaveText,
                handler : function() {
                    if (me.isValid()) {
                        me.fireEvent('save', me.getValues());
                        me.hide();
                    }
                }
            });
        }
        me.addEvents({
            cancel : true,
            save : true
        });

        var oStaticSubitems = [];
        var flexControl = null;
        if (config.staticControls != null) {
            for ( var iIndex = 0; iIndex < config.staticControls.length; iIndex++) {
                var oControl = me.makeControl(config.staticControls[iIndex],false);
                if (oControl != null) {
                    oStaticSubitems.push(oControl);
                    oControl._myTabIndex = -1;
                    if (config.staticControls[iIndex].flex)
                        flexControl = oControl;
                }
            }
        }
        if (config.tabControls != null) {
            this.oTabPanelParent = Ext.create('Ext.tab.Panel', {
                height : this.innerHeight
            });
            this.innerHeight -= 27;
            var tabbedHeight = this.innerHeight;
            for ( var iIndex = 0; iIndex < config.tabControls.length; iIndex++) {
                if (config.tabControls[iIndex].defaultTabOnNew!=null)
                    if (config.tabControls[iIndex].defaultTabOnNew)
                    	this.defaultFirstTabOnNew=iIndex;
                if (config.tabControls[iIndex].defaultTabOnEdit!=null)
                    if (config.tabControls[iIndex].defaultTabOnEdit)
                    	this.defaultFirstTabOnEdit=iIndex;


            	flexControl = null;
                this.innerHeight = tabbedHeight;
                if (config.tabControls[iIndex].content.length > 0) {
                    var oTabPanel = Ext.create('Ext.panel.Panel', {
                        title : config.tabControls[iIndex].tabLabel,
                        border : '0'
                    });
                    this.oTabPanels["" + iIndex]={panel:oTabPanel,title:config.tabControls[iIndex].tabLabel};
                    if (config.tabControls[iIndex].tabDescription!=null) {
                        
                        oTabPanel.add(Ext.create('Ext.form.field.Display', {
                            border : 0,
                            height:18,
                            value:config.tabControls[iIndex].tabDescription,
                            width:770,
                            margin: "7 0 10 5",
                            style: {
                                fontSize: '11px'
                            }
                        }));
                        this.innerHeight -= 35;
                    }
                    for ( var jIndex = 0; jIndex < config.tabControls[iIndex].content.length; jIndex++) {
                        var oControl = me.makeControl(config.tabControls[iIndex].content[jIndex],true);
                        if (oControl != null) {
                            oControl._myTabIndex = iIndex;
                            oTabPanel.add(oControl);
                            if (config.tabControls[iIndex].content[jIndex].flex)
                                flexControl = oControl;
                        }
                    }
                    this.oTabPanelParent.add(oTabPanel);
                    if (flexControl != null)
                        flexControl.setHeight(this.innerHeight);
                }
            }
            oStaticSubitems.push(this.oTabPanelParent);
        } else {
        	this.innerHeight -= 40;
            if (flexControl != null) {
                flexControl.setHeight(this.innerHeight);
            } else {
                    config.height=500-this.innerHeight;
            }
        }
        this.displayBottomText=Ext.create('Ext.form.field.Display', {
            border : 0,
            value:'',
            style: {
                fontSize: '11px'
            },
            padding:'0 0 2 5',
            flex:1
        });
        var oButtons=[this.displayBottomText];
        if (me.buttonSave!=null)
            oButtons.push(me.buttonSave);
        if (me.buttonCancel!=null)
            oButtons.push(me.buttonCancel);
        
        var insideItems;
        if (config.tabControls!=null)
        	insideItems=oStaticSubitems;
        else {
        	insideItems=Ext.create("Ext.panel.Panel", {     
        		bodyPadding : 10,
        		border:"1px",
        		margin:"10 10 10 10",
        		items:oStaticSubitems
        	});
        	insideItems.updateLayout();
        }
        
        
        Ext.apply(me, {
            items : [ {
                xtype : 'panel',
                items : insideItems,
                bodyPadding : 5,
                border : 0,
                dockedItems: [{
                    xtype: 'toolbar',
                    dock: 'bottom',
                    ui: 'footer',
                    defaults: {minWidth: 75},
                    items: oButtons
                }]
            } ]
        }); // Ext.apply

        this.callParent(arguments);

    },

    setValues : function(orgValues) {
    	var values=Ext.JSON.decode(Ext.JSON.encode(orgValues));
        if (values == null) {
            for ( var sKey in this.editControls) {
            	try {
	                if (this.editControls[sKey].reset != null)
	                    this.editControls[sKey].reset();
	                else if (this.editControls[sKey].resetComponent != null)
	                    this.editControls[sKey].resetComponent();
            	}
            	catch (e) {}
            }
        } else {
            for ( var sKey in this.editControls) {
            	try {
            		if (this.oTabPanelParent != null)
	                    if (this.editControls[sKey]._myTabIndex != -1) {
	                        this.oTabPanelParent.doLayout();
	                        this.oTabPanelParent.setActiveTab(this.editControls[sKey]._myTabIndex);
	                    }
	                if (values[sKey] != null) {
	                    this.editControls[sKey].setValue(values[sKey]);
	                } else {
	                    if (this.editControls[sKey].reset != null)
	                        this.editControls[sKey].reset();
	                    else if (this.editControls[sKey].resetComponent != null)
	                        this.editControls[sKey].resetComponent();
	                }
                }
            	catch (e) {}
            }
        }
        if (this.oTabPanelParent != null) {
        	if (orgValues==null) {
        		this.oTabPanelParent.setActiveTab(this.defaultFirstTabOnNew);
        	} else {
        		this.oTabPanelParent.setActiveTab(this.defaultFirstTabOnEdit);
        	}            
            for ( var sKey in this.oTabPanels) {
                this.oTabPanels[sKey].panel.setTitle(this.oTabPanels[sKey].title);
            }

        }
    }, // resetComponent

    isValid : function() {
        var fisValid = true;
        for (var sKey in this.oTabPanels)
            this.oTabPanels[sKey].panel.setTitle(this.oTabPanels[sKey].title);
        
        for ( var sKey in this.editControls) {
            if (!this.editControls[sKey].isValid()) {
                fisValid = false;
                if (this.editControls[sKey]._myTabIndex!=-1)
                    this.oTabPanels["" + this.editControls[sKey]._myTabIndex].panel.setTitle('<span style="color:#ff0000;">' + this.oTabPanels["" + this.editControls[sKey]._myTabIndex].title + '</span>');
            }
        }
        return fisValid;
    },

    getValues : function() {
        var oValues = {};
        for ( var sKey in this.editControls) {
            oValues[sKey] = this.editControls[sKey].getValue();
        }
        return oValues;
    },
    enableControl : function(sControl,fEnable) {
        if (this.editControls[sControl]!=null)
            this.editControls[sControl].setDisabled(!fEnable);
    }
});

